#include <iostream>
#include <fstream>
#include <numeric>
#include <boost/program_options.hpp>

namespace po = boost::program_options;

void processInputStream(std::istream& is) {

    is.seekg(0,is.end);
    auto seq_end = is.tellg();
    std::cout << is.tellg() << " bytes " << std::endl;
    is.seekg(0,is.beg);

    std::array<std::size_t,256> frequencyMap{};

    char buf[10 * 1024];

    while (is.tellg() < seq_end) {
        auto left_2_read = seq_end - is.tellg();
        std::size_t num_bytes = (left_2_read > sizeof(buf))
            ? sizeof(buf) : static_cast<std::size_t>(left_2_read);
        is.read(buf,num_bytes);
        for (int i=0; i<num_bytes; i++) {
            frequencyMap[static_cast<unsigned char>(buf[i])] ++;
        }
    }

    std::size_t total = std::accumulate(frequencyMap.begin(),frequencyMap.end(),static_cast<size_t>(0));
    std::cout << total << std::endl;

    // only for interable containers!!!
    for ( auto&& c : frequencyMap ) {
        std::cout << c << " ";
    }
    std::cout << std::endl;



}

int main(int argc, const char ** argv) {

    std::string inputFile, outputFile;

    po::options_description desc("Program Options");
    desc.add_options()
        ("help", "help message")
        ("input,i", po::value<std::string>(&inputFile), "input file")
        ("output,o", po::value<std::string>(&outputFile), "output file")
    ;

    po::positional_options_description p;
    p.add("input", -1);

    po::variables_map vm;
    po::store(po::command_line_parser(argc,argv).options(desc).positional(p).run(),vm);
    po::notify(vm);

    if (vm.count("help")) {
        std::cout << desc << std::endl;
    }

    if (vm.count("input")) {
        std::cout << "Processing " << inputFile << std::endl;
        try {
            std::ifstream ifs(inputFile,std::ios::binary);
            ifs.exceptions(std::ifstream::badbit | std::ifstream::failbit);
            processInputStream(ifs);
        } catch (const std::ifstream::failure& e) {
            std::cerr << "Input stream error " << e.what() << std::endl;
            return 1;
        }
    } else {
        std::cout << "Process standard input" << std::endl;
        processInputStream(std::cin);
    }

    return 0;
}